-----------------------------------------------------------------------------
--  ns3ms.sql : SQL script for creating nService 3.5 tables
--  (c) 2007 Avensoft. All rights reserved.              
-----------------------------------------------------------------------------
CREATE TABLE ns3_organization
(
    org_id          INT PRIMARY KEY,
    org_name        VARCHAR(50) NOT NULL,
    org_type        TINYINT NOT NULL DEFAULT 2, -- not used
    org_description VARCHAR(250) NULL,
    org_created_on  DATETIME NOT NULL DEFAULT getdate(), 
    org_updated_on  DATETIME NOT NULL DEFAULT getdate(), 
    org_status      TINYINT NOT NULL DEFAULT 1, -- 1=active 2=inactive
    org_deleted     BIT NOT NULL DEFAULT 0
)
GO

-- zone is a physical location. can drive routing of requests.
CREATE TABLE ns3_zone
(
    zone_id         INT PRIMARY KEY, 
    zone_name       VARCHAR(50) NOT NULL,
    zone_description VARCHAR(250) NULL,
    zone_status     TINYINT NOT NULL DEFAULT 1, -- 1=active 2=inactive
    zone_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_site
(
    site_id         INT PRIMARY KEY,
    site_org_id     INT NOT NULL REFERENCES ns3_organization(org_id),
    site_zone_id    INT NULL REFERENCES ns3_zone(zone_id),
    site_name       VARCHAR(50) NOT NULL,
    site_is_hq      BIT NOT NULL DEFAULT 0,
    site_email      VARCHAR(50) NULL,
    site_phone      VARCHAR(20) NULL,
    site_fax        VARCHAR(20) NULL,
    site_addr_l1    VARCHAR(50) NULL,
    site_addr_l2    VARCHAR(50) NULL,  
    site_addr_l3    VARCHAR(50) NULL,
    site_city       VARCHAR(50) NULL,
    site_state      VARCHAR(50) NULL,
    site_zipcode    VARCHAR(20) NULL,
    site_country    VARCHAR(50) NULL,
    site_description VARCHAR(250) NULL,
    site_created_on DATETIME NOT NULL DEFAULT getdate(), 
    site_updated_on DATETIME NOT NULL DEFAULT getdate(), 
    site_status     TINYINT NOT NULL DEFAULT 1, -- 1=active 2=inactive
    site_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_job
(
    job_id          INT PRIMARY KEY,
    job_org_id      INT NOT NULL REFERENCES ns3_organization(org_id), 
    job_name        VARCHAR(50) NOT NULL,
    job_description VARCHAR(250) NULL,
    job_status      TINYINT NOT NULL DEFAULT 1,
    job_deleted     BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_org_unit
(
    ou_id           INT PRIMARY KEY,
    ou_parent_id    INT NULL REFERENCES ns3_org_unit(ou_id),
    ou_name         VARCHAR(50) NOT NULL,
    ou_guid         VARCHAR(50) NULL, -- LDAP
    ou_type         TINYINT NOT NULL DEFAULT 1,
    ou_description  VARCHAR(250) NULL,
    ou_created_on   DATETIME NOT NULL DEFAULT getdate(), 
    ou_updated_on   DATETIME NOT NULL DEFAULT getdate(), 
    ou_status       TINYINT NOT NULL DEFAULT 1, -- 1=active 2=inactive
    ou_deleted      BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_user
(
    usr_id              INT PRIMARY KEY,
    usr_org_id          INT NULL REFERENCES ns3_organization(org_id),
    usr_ou_id           INT NULL REFERENCES ns3_org_unit(ou_id),
    usr_zone_id         INT NULL REFERENCES ns3_zone(zone_id),
    usr_site_id         INT NULL REFERENCES ns3_site(site_id),
    usr_logon_name      VARCHAR(60) NOT NULL UNIQUE,
    usr_password        VARCHAR(70) NULL,
    usr_first_name      VARCHAR(50) NULL,
    usr_last_name       VARCHAR(50) NULL,
    usr_display_name    VARCHAR(50) NOT NULL,
    usr_guid            VARCHAR(50) NULL, -- LDAP
    usr_emp_num         VARCHAR(20) NULL, 
    usr_email           VARCHAR(90) NULL,
    usr_phone           VARCHAR(20) NULL,
    usr_mobile          VARCHAR(20) NULL,
    usr_fax             VARCHAR(20) NULL,
    usr_dept            VARCHAR(30) NULL,
    usr_title           VARCHAR(30) NULL,
    usr_company         VARCHAR(30) NULL,
    usr_office          VARCHAR(30) NULL,
    usr_description     VARCHAR(50) NULL,
    usr_addr_l1         VARCHAR(30) NULL,
    usr_addr_l2         VARCHAR(30) NULL,  
    usr_addr_l3         VARCHAR(30) NULL,
    usr_city            VARCHAR(30) NULL,
    usr_state           VARCHAR(30) NULL,
    usr_zipcode         VARCHAR(20) NULL,
    usr_country         VARCHAR(30) NULL,
    usr_theme           VARCHAR(20) NULL, 
    usr_role_in_org     TINYINT NOT NULL DEFAULT 1,  -- 1=user; 2=admin
    usr_status          TINYINT NOT NULL DEFAULT 1, 
    usr_deleted         BIT NOT NULL DEFAULT 0,
    usr_last_login      DATETIME NULL,
    usr_created_on      DATETIME NOT NULL DEFAULT getdate(), 
    usr_updated_on      DATETIME NULL,
    usr_online          BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_user_org_id ON ns3_user(usr_org_id)
GO
CREATE INDEX ns3_idx_user_site_id ON ns3_user(usr_site_id)
GO
CREATE INDEX ns3_idx_user_status ON ns3_user(usr_status)
GO

CREATE TABLE ns3_group
(
    grp_id          INT PRIMARY KEY,
    grp_ou_id       INT NULL REFERENCES ns3_org_unit(ou_id),
    grp_name        VARCHAR(50) NOT NULL,
    grp_guid        VARCHAR(50) NULL, -- LDAP
    grp_ldap_path   VARCHAR(50) NULL, -- LDAP
    grp_description VARCHAR(250) NULL, 
    grp_default     BIT NOT NULL DEFAULT 0, 
    grp_status      TINYINT NOT NULL DEFAULT 1, -- 1=active 2=inactive
    grp_deleted     BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_ou_admin_group
(
    oag_ou_id       INT NOT NULL REFERENCES ns3_org_unit(ou_id),
    oag_grp_id      INT NOT NULL REFERENCES ns3_group(grp_id), 
    CONSTRAINT ns3_pk_ou_admin_group PRIMARY KEY (oag_ou_id, oag_grp_id)
)
GO 

CREATE TABLE ns3_user_in_group
(
    uig_usr_id      INT NOT NULL REFERENCES ns3_user(usr_id),
    uig_grp_id      INT NOT NULL REFERENCES ns3_group(grp_id), 
    CONSTRAINT ns3_pk_user_in_group PRIMARY KEY (uig_usr_id, uig_grp_id)
)
GO 

CREATE TABLE ns3_group_in_group
(
    gig_grp_id      INT NOT NULL REFERENCES ns3_group(grp_id),
    gig_in_grp_id   INT NOT NULL REFERENCES ns3_group(grp_id), 
    CONSTRAINT ns3_pk_group_in_group PRIMARY KEY (gig_grp_id, gig_in_grp_id)
)
GO 

CREATE TABLE ns3_product
(
    pd_id           INT PRIMARY KEY,
    pd_parent_id    INT NULL REFERENCES ns3_product(pd_id),
    pd_user_grp     INT NULL REFERENCES ns3_group(grp_id),
    pd_tech_grp     INT NULL REFERENCES ns3_group(grp_id),
    pd_admin_grp    INT NULL REFERENCES ns3_group(grp_id),
    pd_name         VARCHAR(50) NOT NULL,
    pd_category     BIT NOT NULL DEFAULT 0,
    pd_item_number  VARCHAR(50) NULL, 
    pd_maker_id     INT NULL REFERENCES ns3_organization(org_id),
    pd_seller_id    INT NULL REFERENCES ns3_organization(org_id),
    pd_keywords     VARCHAR(200) NULL,
    pd_description  VARCHAR(250) NULL,
    pd_status       TINYINT NOT NULL DEFAULT 1,  -- 1=active, 2=inactive
    pd_deleted      BIT NOT NULL DEFAULT 0
)
GO

-- e.g. Gateway 2000, Hard Disk 1, Maxtor 40G
CREATE TABLE ns3_product_part
(
    part_id         INT PRIMARY KEY,
    part_product_id INT NOT NULL REFERENCES ns3_product(pd_id), 
    part_label      VARCHAR(50) NOT NULL,  
    part_part_id    INT NOT NULL REFERENCES ns3_product(pd_id), 
    part_count      TINYINT NOT NULL DEFAULT 1,
    part_order      TINYINT NOT NULL DEFAULT 1
)
GO

-- Extend the concept of computers in active directory 
CREATE TABLE ns3_asset
(
    as_id               INT PRIMARY KEY,
    as_org_id           INT NULL REFERENCES ns3_organization(org_id), 
    as_ou_id            INT NULL REFERENCES ns3_org_unit(ou_id),
    as_site_id          INT NULL REFERENCES ns3_site(site_id), 
    as_usr_id           INT NULL REFERENCES ns3_user(usr_id), 
    as_pd_id            INT NULL REFERENCES ns3_product(pd_id), 
    as_name             VARCHAR(50) NULL,
    as_guid             VARCHAR(50) NULL,
    as_type             TINYINT NOT NULL DEFAULT 1, -- 1=general 2=computer 3=printer
    as_description      VARCHAR(250) NULL, 
    as_serial_num       VARCHAR(50) NULL,
    as_num_user         INT NOT NULL DEFAULT 1,
    as_product_key      VARCHAR(50) NULL,
    as_purchase_date    DATETIME NULL,
    as_regist_date      DATETIME NULL,
    as_maint_exp        DATETIME NULL,
    as_warranty_exp     DATETIME NULL,
    as_warranty_type    VARCHAR(30) NULL,
    as_service_tag      VARCHAR(50) NULL,
    as_location         VARCHAR(250) NULL,
    as_pwd              VARCHAR(20) NULL,
    as_cpu              VARCHAR(30) NULL,
    as_ram              VARCHAR(30) NULL,
    as_harddisk         VARCHAR(30) NULL,
    as_cd_dvd           VARCHAR(30) NULL,
    as_monitor          VARCHAR(30) NULL,
    as_os               VARCHAR(30) NULL,
    as_service_pack     VARCHAR(30) NULL,
    as_office_ver       VARCHAR(30) NULL,
    as_bios             VARCHAR(30) NULL,
    as_firmware_ver     VARCHAR(50) NULL,
    as_r_acc_type       VARCHAR(50) NULL,
    as_r_acc_info       VARCHAR(50) NULL,
    as_r_acc_pwd        VARCHAR(20) NULL,
    as_status           TINYINT NOT NULL DEFAULT 1,  
                        -- 1=in use, 2=in repair, 3=idle 
    as_deleted          BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_asset_org_id ON ns3_asset(as_org_id)
GO
CREATE INDEX ns3_idx_asset_site_id ON ns3_asset(as_site_id)
GO
CREATE INDEX ns3_idx_asset_usr_id ON ns3_asset(as_usr_id)
GO
CREATE INDEX ns3_idx_asset_pd_id ON ns3_asset(as_pd_id)
GO

CREATE TABLE ns3_as_note
(
    asn_id          INT PRIMARY KEY,
    asn_as_id       INT NOT NULL REFERENCES ns3_asset(as_id) 
                    ON DELETE CASCADE, 
    asn_added_by    INT NULL REFERENCES ns3_user(usr_id), 
    asn_added_on    DATETIME NOT NULL DEFAULT getdate(),
    asn_type        TINYINT NOT NULL DEFAULT 1, 
                    -- 1=public; 2=internal; 3=audit
    asn_format      TINYINT NOT NULL DEFAULT 1, -- 1=plain text; 2=html;
    asn_detail      TEXT NULL,
    asn_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_as_note_as_id ON ns3_as_note(asn_as_id)
GO

CREATE TABLE ns3_asn_file
(
    anf_id          INT PRIMARY KEY,
    anf_asn_id      INT NOT NULL REFERENCES ns3_as_note(asn_id) 
                    ON DELETE CASCADE, 
    anf_name        VARCHAR(250) NOT NULL,
    anf_mime_type   VARCHAR(30) NOT NULL, 
    anf_updated_on  DATETIME NOT NULL DEFAULT getdate(), 
    anf_size        INT NOT NULL,
    anf_content     IMAGE NULL,
    anf_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_asn_file_asn_id ON ns3_asn_file(anf_asn_id)
GO

-- enterprise services can be active directory objects to be looked up too
CREATE TABLE ns3_service
(
    srv_id          INT PRIMARY KEY,
    srv_p1          INT NULL REFERENCES ns3_service(srv_id),
    srv_p2          INT NULL REFERENCES ns3_service(srv_id),
    srv_p3          INT NULL REFERENCES ns3_service(srv_id),
    srv_p4          INT NULL REFERENCES ns3_service(srv_id),
    srv_p5          INT NULL REFERENCES ns3_service(srv_id),
    srv_user_grp    INT NOT NULL REFERENCES ns3_group(grp_id),
    srv_tech1_grp   INT NOT NULL REFERENCES ns3_group(grp_id),
    srv_tech2_grp   INT NOT NULL REFERENCES ns3_group(grp_id),
    srv_admin_grp   INT NOT NULL REFERENCES ns3_group(grp_id),
    srv_inherit_grp BIT NOT NULL DEFAULT 1, 
    srv_name        VARCHAR(50) NOT NULL,
    srv_keywords    VARCHAR(200) NULL,
    srv_description VARCHAR(250) NULL,
    -- 1=all products, 2=non-product related, 3=ns3_service_for_product table
    -- eg. techinical support, billing questions, large size tv installation.
    srv_product_rel TINYINT NOT NULL DEFAULT 1,
    srv_order_rel   BIT NOT NULL DEFAULT 0, 
    srv_email_addr  VARCHAR(50) NULL,
    srv_smtp_user   VARCHAR(50) NULL,
    srv_smtp_pwd    VARCHAR(50) NULL,
    srv_pop3_user   VARCHAR(50) NULL,
    srv_pop3_pwd    VARCHAR(50) NULL,
    srv_options     VARCHAR(50) NULL, 
    srv_status      TINYINT NOT NULL DEFAULT 1,  -- 1=active, 2=inactive 
    srv_deleted     BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_service_product
(
    sp_id           INT PRIMARY KEY,
    sp_srv_id       INT NOT NULL REFERENCES ns3_service(srv_id), 
    sp_pd_id        INT NOT NULL REFERENCES ns3_product(pd_id) 
)
GO

CREATE TABLE ns3_service_rate
(
    rate_id         INT PRIMARY KEY,
    rate_name       VARCHAR(20) NOT NULL UNIQUE,
    rate_srv_id     INT NULL REFERENCES ns3_service(srv_id), 
    rate_usr_id     INT NULL REFERENCES ns3_user(usr_id), 
    rate_value      NUMERIC(8, 2) NOT NULL DEFAULT 0,
    rate_is_def     BIT NOT NULL DEFAULT 0,
    rate_status     TINYINT NOT NULL DEFAULT 1,
    rate_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_req_status
(
    rs_id           INT PRIMARY KEY,
    rs_srv_id       INT NOT NULL REFERENCES ns3_service(srv_id), 
    rs_name         VARCHAR(50) NOT NULL,
    rs_sequence     TINYINT NOT NULL DEFAULT 5, -- 1=New; 9=Closed
    rs_description  VARCHAR(250) NULL,
    rs_deleted      BIT NOT NULL DEFAULT 0
)
GO
  
CREATE TABLE ns3_req_priority
(
    rp_id           INT PRIMARY KEY,
    rp_srv_id       INT NOT NULL REFERENCES ns3_service(srv_id), 
    rp_name         VARCHAR(50) NOT NULL,
    rp_default      BIT NOT NULL DEFAULT 0,
    rp_description  VARCHAR(250) NULL,
    rp_deleted      BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_service_request
(
    sr_id           INT PRIMARY KEY,
    sr_status_id    INT NULL REFERENCES ns3_req_status(rs_id), 
    sr_service_id   INT NULL REFERENCES ns3_service(srv_id), 
    sr_product_id   INT NULL REFERENCES ns3_product(pd_id), 
    sr_asset_id     INT NULL REFERENCES ns3_asset(as_id), 
    sr_priority_id  INT NULL REFERENCES ns3_req_priority(rp_id), 
    sr_vendor_id    INT NULL REFERENCES ns3_organization(org_id), 
    sr_order_num    VARCHAR(20) NULL, -- specific to vendor_id
    sr_job_id       INT NULL REFERENCES ns3_job(job_id), -- client accounting
    sr_submitter    INT NULL REFERENCES ns3_user(usr_id), 
    sr_assigned_to  INT NULL REFERENCES ns3_user(usr_id), 
    sr_closed_by    INT NULL REFERENCES ns3_user(usr_id), 
    sr_submitter_cc VARCHAR(250) NULL, 
    sr_submitted_on DATETIME NOT NULL,
    sr_assigned_on  DATETIME NULL,
    sr_closed_on    DATETIME NULL,
    sr_due_on       DATETIME NULL,
    sr_responded_on DATETIME NULL,
    sr_escalated_on DATETIME NULL,
    sr_updated_on   DATETIME NOT NULL,
    sr_subject      VARCHAR(250) NOT NULL,
    sr_bill_hour    NUMERIC(10, 2) NULL DEFAULT 0,
    sr_nbill_hour   NUMERIC(10, 2) NULL DEFAULT 0,
    sr_turn_around  INT NOT NULL DEFAULT 0,
    sr_due_reminded BIT NOT NULL DEFAULT 0,
    sr_deleted      BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_service_request_status_id 
    ON ns3_service_request(sr_status_id)
GO
CREATE INDEX ns3_idx_service_request_submitter 
    ON ns3_service_request(sr_submitter)
GO
CREATE INDEX ns3_idx_service_request_assigned_to 
    ON ns3_service_request(sr_assigned_to)
GO

CREATE TABLE ns3_call
(
    call_id         INT PRIMARY KEY,
    call_caller     INT NULL REFERENCES ns3_user(usr_id), 
    call_receiver   INT NULL REFERENCES ns3_user(usr_id), 
    call_start      DATETIME NOT NULL DEFAULT getdate(),
    call_end        DATETIME NULL,
    call_status     TINYINT NOT NULL DEFAULT 1, 
    call_last_sr    INT NULL REFERENCES ns3_service_request(sr_id), 
    call_note       VARCHAR(250) NULL,
    call_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_sr_note
(
    srn_id          INT PRIMARY KEY,
    srn_sr_id       INT NOT NULL REFERENCES ns3_service_request(sr_id) 
                    ON DELETE CASCADE, 
    srn_call_id     INT NULL REFERENCES ns3_call(call_id), 
    srn_added_by    INT NULL REFERENCES ns3_user(usr_id), 
    srn_added_on    DATETIME NOT NULL DEFAULT getdate(),
    srn_bill_rate   INT NULL REFERENCES ns3_service_rate(rate_id),
    srn_bill_hour   NUMERIC(10, 2) NOT NULL DEFAULT 0,
    srn_nbill_hour  NUMERIC(10, 2) NOT NULL DEFAULT 0,
    srn_type        TINYINT NOT NULL DEFAULT 1, 
                    -- 1=public; 2=internal; 3=audit
    srn_format      TINYINT NOT NULL DEFAULT 1, -- 1=plain text; 2=html;
    srn_detail      TEXT NULL,
    srn_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_sr_note_sr_id ON ns3_sr_note(srn_sr_id)
GO

CREATE TABLE ns3_srn_file
(
    snf_id          INT PRIMARY KEY,
    snf_srn_id      INT NOT NULL REFERENCES ns3_sr_note(srn_id) 
                    ON DELETE CASCADE, 
    snf_name        VARCHAR(250) NOT NULL,
    snf_mime_type   VARCHAR(30) NOT NULL, 
    snf_updated_on  DATETIME NOT NULL DEFAULT getdate(), 
    snf_size        INT NOT NULL,
    snf_content     IMAGE NULL,
    snf_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_srn_file_srn_id ON ns3_srn_file(snf_srn_id)
GO

CREATE TABLE ns3_sr_routing_rule
(
    rule_id         INT PRIMARY KEY,
    rule_name       VARCHAR(50) NOT NULL, 
    rule_description VARCHAR(250) NULL, 
    rule_srv_id     INT NULL REFERENCES ns3_service(srv_id), 
    rule_zone_id    INT NULL REFERENCES ns3_zone(zone_id), 
    rule_ou_id      INT NULL REFERENCES ns3_org_unit(ou_id), 
    rule_site_id    INT NULL REFERENCES ns3_site(site_id), 
    rule_pd_id      INT NULL REFERENCES ns3_product(pd_id), 
    rule_priority_id INT NULL REFERENCES ns3_req_priority(rp_id),  
    rule_age_limit  INT NOT NULL DEFAULT 0, 

    -- Submitted = 1, Assigned = 2, NewInfoSubmitted = 3, 
    -- Responded = 4, PrivateNoteAdded = 5, ClosedByTech = 6
    -- ClosedBySubmitter = 7
    rule_event      TINYINT NOT NULL DEFAULT 1, 
    
    -- Assign = 1, SendConfirmation = 2, SendNotification = 3
    rule_action     TINYINT NOT NULL DEFAULT 1, 
    
    -- 1=act_usr_id; 2=submitter; 3=assigned_to, 4=closed_by
    rule_act_usr_var TINYINT NOT NULL DEFAULT 1, 
    rule_act_usr_id INT NULL REFERENCES ns3_user(usr_id), 
    rule_act_grp_id INT NULL REFERENCES ns3_group(grp_id), 
    rule_status     TINYINT NOT NULL DEFAULT 1,  -- 1=active, 2=inactive 
    rule_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_req_query 
(
    rq_id               INT PRIMARY KEY,
    rq_usr_id           INT NULL REFERENCES ns3_user(usr_id), 
    rq_usr_grp          INT NULL REFERENCES ns3_group(grp_id), 
    rq_name             VARCHAR(50) NOT NULL,
    rq_type             TINYINT NOT NULL DEFAULT 1, 
    rq_status_ids       VARCHAR(50) NULL, 
    rq_service_id       INT NULL REFERENCES ns3_service(srv_id), 
    rq_product_id       INT NULL, -- could be 1 to explicitly query IS NULL 
    rq_asset_id         INT NULL, -- could be 1 to explicitly query IS NULL 
    rq_priority_ids     VARCHAR(50) NULL, 
    rq_submitter_org    INT NULL, -- 1 = No Org, i.e. IS NULL = Individual
    rq_submitter_site   INT NULL, 
    rq_submitter        INT NULL, -- 1 = No One, i.e. IS NULL, not don't care
    rq_assigned_to      INT NULL, -- 1 = No One, i.e. IS NULL
    rq_closed_by        INT NULL, -- 1 = No One, i.e. IS NULL
    rq_job_id           INT NULL, -- could be 1 to explicitly query IS NULL 
    rq_keywords         VARCHAR(50) NULL, 
    rq_submitted_on     TINYINT NULL, 
    rq_submitted_on1    DATETIME NULL, 
    rq_submitted_on2    DATETIME NULL, 
    rq_due_on           TINYINT NULL, 
    rq_due_on1          DATETIME NULL, 
    rq_due_on2          DATETIME NULL, 
    rq_closed_on        TINYINT NULL, 
    rq_closed_on1       DATETIME NULL, 
    rq_closed_on2       DATETIME NULL, 
    rq_bill_hour1       NUMERIC(10, 2), 
    rq_bill_hour2       NUMERIC(10, 2), 
    rq_nbill_hour1      NUMERIC(10, 2), 
    rq_nbill_hour2      NUMERIC(10, 2), 
    rq_turn_around1     INT NULL, 
    rq_turn_around2     INT NULL, 
    rq_max_row          SMALLINT NOT NULL DEFAULT 0, 
    rq_row_per_page     SMALLINT NOT NULL DEFAULT 15, 
    rq_order_by1        VARCHAR(30), 
    rq_order_by1_asc    BIT NULL, 
    rq_order_by2        VARCHAR(30),  
    rq_order_by2_asc    BIT NULL,
    rq_select           VARCHAR(250) NULL, 
    rq_column_labels    VARCHAR(250) NULL, 
    rq_where            VARCHAR(200) NULL, 
    rq_orderby          VARCHAR(50) NULL, 
    rq_group_by1        VARCHAR(30) NULL, 
    rq_group_by1_type   TINYINT NOT NULL DEFAULT 3,
    rq_group_by2        VARCHAR(30) NULL, 
    rq_group_by2_type   TINYINT NOT NULL DEFAULT 3,
    rq_top_group        TINYINT NOT NULL DEFAULT 0, 
    rq_description      VARCHAR(250) NULL,
    rq_ref_date         DATETIME NULL,
    rq_deleted          BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_task
(
    task_id         INT PRIMARY KEY,
    task_usr_id     INT NOT NULL REFERENCES ns3_user(usr_id), 
    task_sr_id      INT NULL REFERENCES ns3_service_request(sr_id)
                    ON DELETE CASCADE, 
    task_type       TINYINT NOT NULL DEFAULT 1, -- 1=appointment; 2=task; 
    task_title      VARCHAR(50) NOT NULL,
    task_start      DATETIME NULL,
    task_end        DATETIME NULL, -- due date for task
    task_len        INT NULL DEFAULT 0,
    task_note       VARCHAR(250), 
    task_deleted    BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_kbase
(
    kb_id           INT PRIMARY KEY,
    kb_parent_id    INT NULL REFERENCES ns3_kbase(kb_id),
    kb_user_grp     INT NOT NULL REFERENCES ns3_group(grp_id),
    kb_tech_grp     INT NOT NULL REFERENCES ns3_group(grp_id),
    kb_admin_grp    INT NOT NULL REFERENCES ns3_group(grp_id),
    kb_inherit_grp  BIT NOT NULL DEFAULT 1,
    kb_type         TINYINT NOT NULL DEFAULT 1,
    kb_name         VARCHAR(100) NOT NULL,
    kb_keywords     VARCHAR(200) NULL,
    kb_description  VARCHAR(250) NULL,
    kb_deleted      BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_kbase_parent_id ON ns3_kbase(kb_parent_id)
GO

CREATE TABLE ns3_article
(
    art_id          INT PRIMARY KEY,
    art_kb_id       INT NOT NULL REFERENCES ns3_kbase(kb_id)
                    ON DELETE CASCADE,
    art_type        TINYINT NOT NULL DEFAULT 1,
    art_total_votes INT NOT NULL DEFAULT 0,
    art_up_votes    INT NOT NULL DEFAULT 0,
    art_views       INT NOT NULL DEFAULT 0,
    art_title       VARCHAR(100) NOT NULL,
    art_keywords    VARCHAR(200) NULL,
    art_description VARCHAR(250) NULL,
    art_updated_on  DATETIME NOT NULL DEFAULT getdate(),
    art_updated_by  INT NULL REFERENCES ns3_user(usr_id),
    art_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_article_kb_id ON ns3_article(art_kb_id)
GO

CREATE TABLE ns3_article_note
(
    an_id           INT PRIMARY KEY,
    an_art_id       INT NOT NULL REFERENCES ns3_article(art_id)
                    ON DELETE CASCADE, 
    an_added_by     INT NULL REFERENCES ns3_user(usr_id), 
    an_added_on     DATETIME NOT NULL DEFAULT getdate(),
    an_type         TINYINT NOT NULL DEFAULT 1, 
                    -- 1=article content; 2=feedback
    an_format       TINYINT NOT NULL DEFAULT 1, -- 1=plain text; 2=html;
    an_detail       TEXT NULL, 
    an_deleted      BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_article_note_art_id ON ns3_article_note(an_art_id)
GO

CREATE TABLE ns3_article_file
(
    af_id           INT PRIMARY KEY,
    af_an_id        INT NOT NULL REFERENCES ns3_article_note(an_id) 
                    ON DELETE CASCADE,
    af_file_name    VARCHAR(100) NOT NULL,
    af_mime_type    VARCHAR(30) NOT NULL, 
    af_updated_on   DATETIME NOT NULL DEFAULT getdate(), 
    af_file_size    INT NOT NULL,
    af_file_data    IMAGE NOT NULL, 
    af_deleted      BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_article_file_an_id ON ns3_article_file(af_an_id)
GO

CREATE TABLE ns3_article_keyword
(
    ak_id           INT PRIMARY KEY, 
    ak_art_id       INT NOT NULL REFERENCES ns3_article(art_id)
                    ON DELETE CASCADE, 
    ak_keyword      VARCHAR(50) NOT NULL,
    ak_count        SMALLINT NOT NULL DEFAULT 1
)
GO

CREATE TABLE ns3_wizard
(
    wiz_id              INT PRIMARY KEY,
    wiz_parent_id       INT NULL REFERENCES ns3_wizard(wiz_id),
    wiz_kb_id           INT NOT NULL REFERENCES ns3_kbase(kb_id)
                        ON DELETE CASCADE,
    wiz_answer_to_last  VARCHAR(200) NULL,
    wiz_goto_url        VARCHAR(200) NULL, 
    wiz_explanation     TEXT NULL,
    wiz_next_question   VARCHAR(200) NULL, 
    wiz_deleted         BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_wizard_parent_id ON ns3_wizard(wiz_parent_id)
GO
CREATE INDEX ns3_idx_wizard_kb_id ON ns3_wizard(wiz_kb_id)
GO

CREATE TABLE ns3_wiz_file
(
    wf_id           INT PRIMARY KEY,
    wf_wiz_id       INT NOT NULL REFERENCES ns3_wizard(wiz_id) 
                    ON DELETE CASCADE, 
    wf_name         VARCHAR(250) NOT NULL,
    wf_mime_type    VARCHAR(30) NOT NULL, 
    wf_updated_on   DATETIME NOT NULL DEFAULT getdate(), 
    wf_size         INT NOT NULL,
    wf_content      IMAGE NULL,
    wf_deleted      BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_wiz_file_wiz_id ON ns3_wizard(wiz_id)
GO

CREATE TABLE ns3_product_article
(
    pa_pd_id    INT NOT NULL REFERENCES ns3_product(pd_id) 
                ON DELETE CASCADE, 
    pa_art_id   INT NOT NULL REFERENCES ns3_article(art_id) 
                ON DELETE CASCADE,
    CONSTRAINT ns3_pk_product_article PRIMARY KEY (pa_pd_id, pa_art_id)
)
GO

CREATE TABLE ns3_product_wizard
(
    pw_pd_id    INT NOT NULL REFERENCES ns3_product(pd_id)
                ON DELETE CASCADE, 
    pw_wiz_id   INT NOT NULL REFERENCES ns3_wizard(wiz_id)
                ON DELETE CASCADE,
    CONSTRAINT ns3_pk_product_wizard PRIMARY KEY (pw_pd_id, pw_wiz_id)
)
GO

CREATE TABLE ns3_service_article
(
    sa_srv_id   INT NOT NULL REFERENCES ns3_service(srv_id)
                ON DELETE CASCADE, 
    sa_art_id   INT NOT NULL REFERENCES ns3_article(art_id)
                ON DELETE CASCADE, 
    CONSTRAINT ns3_pk_service_article PRIMARY KEY (sa_srv_id, sa_art_id)
)
GO

CREATE TABLE ns3_service_wizard
(
    sw_srv_id   INT NOT NULL REFERENCES ns3_service(srv_id)
                ON DELETE CASCADE, 
    sw_wiz_id   INT NOT NULL REFERENCES ns3_wizard(wiz_id)
                ON DELETE CASCADE, 
    CONSTRAINT ns3_pk_service_wizard PRIMARY KEY (sw_srv_id, sw_wiz_id)
)
GO

-- em_folder: 1=draft; 2=outbox; 3=sent; 4=inbox; 5=deleted; 
-- em_srv_id needed to match service that the email is submitted for.
-- Can't use em_from because it could be Cc'ed.
CREATE TABLE ns3_email
(
    em_id           INT PRIMARY KEY,
    em_srv_id       INT NULL REFERENCES ns3_service(srv_id), 
    em_created_on   DATETIME NOT NULL,
    em_folder       INT NOT NULL DEFAULT 1, 
    em_from         VARCHAR(50) NOT NULL, 
    em_to           VARCHAR(250) NOT NULL, 
    em_cc           VARCHAR(250) NULL, 
    em_bcc          VARCHAR(250) NULL, 
    em_subject      VARCHAR(250) NOT NULL, 
    em_date         DATETIME NULL,
    em_priority     VARCHAR(20) NULL,
    em_headers      VARCHAR(250) NULL, 
    em_content_type VARCHAR(50) NULL,
    em_body         TEXT NOT NULL
)
GO

CREATE TABLE ns3_email_file
(
    emf_id          INT PRIMARY KEY,
    emf_em_id       INT NOT NULL REFERENCES ns3_email(em_id) ON DELETE CASCADE, 
    emf_name        VARCHAR(250) NOT NULL,
    emf_mime_type   VARCHAR(30) NOT NULL, 
    emf_updated_on  DATETIME NOT NULL DEFAULT getdate(), 
    emf_size        INT NOT NULL,
    emf_content     IMAGE NULL,
    emf_deleted     BIT NOT NULL DEFAULT 0
)
GO
CREATE INDEX ns3_idx_email_file_em_id ON ns3_email_file(emf_em_id)
GO

CREATE TABLE ns3_email_template
(
    emt_srv_id      INT NOT NULL REFERENCES ns3_service(srv_id)
                    ON DELETE CASCADE, 
    emt_event       TINYINT NOT NULL,  -- maps to rule_event
    emt_action      TINYINT NOT NULL,  -- maps to rule_action
    emt_subject     VARCHAR(200) NOT NULL,
    emt_body        TEXT NOT NULL,
    emt_html_body   TEXT NULL,
    emt_updated_on  DATETIME NOT NULL DEFAULT getdate(),
    CONSTRAINT ns3_pk_email_template 
                    PRIMARY KEY (emt_srv_id, emt_event, emt_action)
)
GO

CREATE TABLE ns3_email_gate
(
    emg_addr        VARCHAR(50) PRIMARY KEY, 
    emg_date        DATETIME NOT NULL,
    emg_count       INT NOT NULL DEFAULT 0,
    emg_option      INT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_event_log
(
    el_id           INT PRIMARY KEY,
    el_type         TINYINT NOT NULL DEFAULT 1,
    el_created_on   DATETIME NOT NULL,
    el_subject      VARCHAR(250) NOT NULL, 
    el_detail       TEXT NOT NULL,
    el_deleted      BIT NOT NULL DEFAULT 0
)
GO

CREATE TABLE ns3_registry
(
    reg_user        VARCHAR(50) NOT NULL,
    reg_section     VARCHAR(50) NOT NULL,
    reg_key         VARCHAR(50) NOT NULL,
    reg_value       VARCHAR(250) NULL,
    CONSTRAINT ns3_pk_registry PRIMARY KEY (reg_user, reg_section, reg_key)
)
GO

INSERT INTO ns3_organization(org_id, org_name) 
    VALUES(90, 'My Organization')  -- hard coded self org ID 90
GO
INSERT INTO ns3_site(site_id, site_org_id, site_name, site_is_hq) 
    VALUES(90, 90, 'First Site', 1)
GO
INSERT INTO ns3_org_unit(ou_id, ou_name) 
    VALUES(90, 'Root Organizational Unit')
GO

-- hard coded sys admin user ID 90
INSERT INTO ns3_user
(
    usr_id, usr_logon_name, usr_display_name, usr_org_id, usr_site_id, usr_ou_id
)
VALUES(90, 'ns3.admin', 'nService Administrator', 90, 90, 90)  
GO
-- hard coded sole anonymous visitor user ID 91
INSERT INTO ns3_user(usr_id, usr_logon_name, usr_display_name)
	VALUES(91, 'ns3.anonymous', 'Anonymous User')  
GO

-- hard coded sole sys admin group ID 90
INSERT INTO ns3_group(grp_id, grp_name, grp_ou_id) 
    VALUES(90, 'System Administrators', 90) 
GO
INSERT INTO ns3_group(grp_id, grp_name, grp_ou_id) 
    VALUES(91, 'Service Administrators', 90)
GO
INSERT INTO ns3_group(grp_id, grp_name, grp_ou_id) 
    VALUES(92, 'Service Technicians', 90)
GO
INSERT INTO ns3_group(grp_id, grp_name, grp_default, grp_ou_id) 
    VALUES(93, 'Everyone', 1, 90)
GO

INSERT INTO ns3_user_in_group(uig_usr_id, uig_grp_id) 
    VALUES(90, 90)
GO
INSERT INTO ns3_user_in_group(uig_usr_id, uig_grp_id) 
    VALUES(90, 91)
GO
INSERT INTO ns3_user_in_group(uig_usr_id, uig_grp_id) 
    VALUES(90, 92)
GO
INSERT INTO ns3_ou_admin_group(oag_ou_id, oag_grp_id)
    VALUES(90, 91)
GO

INSERT INTO ns3_service(srv_id, srv_name, srv_product_rel, 
    srv_inherit_grp, srv_user_grp, srv_tech1_grp, srv_tech2_grp, srv_admin_grp) 
    VALUES(90, 'Services', 0, 0, 93, 92, 91, 91)
GO

INSERT INTO ns3_req_status(rs_id, rs_name, rs_srv_id, rs_sequence)
    VALUES(90, 'Open', 90, 1);
GO
INSERT INTO ns3_req_status(rs_id, rs_name, rs_srv_id, rs_sequence)
    VALUES(91, 'Closed', 90, 9);
GO

INSERT INTO ns3_req_priority(rp_id, rp_name, rp_srv_id)
    VALUES(90, 'High', 90);
GO
INSERT INTO ns3_req_priority(rp_id, rp_name, rp_srv_id, rp_default)
    VALUES(91, 'Normal', 90, 1);
GO
INSERT INTO ns3_req_priority(rp_id, rp_name, rp_srv_id)
    VALUES(92, 'Low', 90);
GO

INSERT INTO ns3_product(pd_id, pd_category, pd_name, 
                        pd_user_grp, pd_tech_grp, pd_admin_grp) 
    VALUES(90, 1, 'Products', 93, 92, 91)
GO

-- make kb root 90 to make data upgrade from HD2, etc easier
INSERT INTO ns3_kbase(kb_id, kb_name, kb_inherit_grp, 
                      kb_user_grp, kb_tech_grp, kb_admin_grp)
    VALUES(90, 'Knowledge Base', 0, 93, 92, 91)
GO
INSERT INTO ns3_kbase(kb_id, kb_parent_id, kb_name, kb_inherit_grp, 
                      kb_user_grp, kb_tech_grp, kb_admin_grp)
    VALUES(91, 90, 'nService3', 0, 92, 91, 90)
GO
INSERT INTO ns3_article(art_id, art_kb_id, art_title, art_description)
    VALUES(90, 91, 'nService3 Manual', '../Help/index.htm')
GO

-- hard coded welcome wizard ID 90
INSERT INTO ns3_wizard(wiz_id, wiz_kb_id, wiz_answer_to_last, 
                       wiz_next_question)
    VALUES(90, 90, 'Welcome Wizard', 'How may we assist you?');  
GO    
INSERT INTO ns3_wizard(wiz_id, wiz_parent_id, wiz_kb_id, wiz_answer_to_last, 
                       wiz_explanation)
VALUES
(
    91, 90, 90, 'I need help', 
    'Please click the Services, Products and Knowledge link above ' +
    'to check if there is information there that can help you. ' +
    'If not, please log on and submit a service request. ' +
    'If you do not have an account yet, ' +
    'please click the Sign Up link above to create one. '
)  
GO    
-- End of file